package biggests.clustering;

/**
 * <p>Title: Hierarchical Cluster Tree Node</p>
 *
 * <p>Description: <code>ClusterTreeNode</code>'s class describes a single
 *                 node in a tree created by hierarchical clustering.
 *                 The tree can be represented by an array of n
 *                 <code>ClusterTreeNode</code>s objects, where n is the
 *                 number of clustered elements minus one. The integers
 *                 left and right refer to the two elements or subnodes
 *                 that are joined in this node. The original elements
 *                 are numbered 0..nelements-1, and the nodes
 *                 -1..-(nelements-1). Distance contains the distance
 *                 between the two subnodes that were joined.
 * <br/>
 *                 This class implements Node struct defined in
 *                 the C source code of Cluster 3.0 (version 1.36) by
 *                 Michiel de Hoon, while at the Laboratory of DNA
 *                 Information Analysis, Human Genome Center, Institute
 *                 of Medical Science, University of Tokyo, Japan. The
 *                 original definition was not changed, so as to maintain
 *                 some direct correspondence between the two concepts
 *                 (this class and the original Node struct).
 * <br/>
 *                 Cluster 3.0 is an enhanced version of Cluster,
 *                 which was originally developed by Michael Eisen
 *                 while at Stanford University.
 * </p>
 *
 * <p>Copyright:   Copyright (C) 2007 Joana Gonçalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 2 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 *
 *                 GNU General Public License also complies with the terms of
 *                 Cluster/TreeView
 *                 <a href="http://rana.lbl.gov/EisenSoftwareSource.htm">
 *                 original license</a>
 * </p>
 *
 * @author Joana P. Gonçalves
 */
public class ClusterTreeNode {
    /**
     * The index of the element or node that is joined to the
     * element/node referred by the index in <code>right</code>.
     */
    private int left;

    /**
     * The index of the element or node that is joined to the
     * element/node referred by the index in <code>left</code>.
     */
    private int right;

    /**
     * The distance between left and right, i.e., the two subnodes
     * (elements or nodes) that are joined by this node.
     */
    private double distance;

    /**
     * Creates a new <code>ClusterTreeNode</code> with <code>left</code>,
     * <code>right</code> and <code>distance</code> attributes initialized
     * to 0.
     */
    public ClusterTreeNode() {
        this(0,0,0);
    }

    /**
     * Creates a new <code>ClusterTreeNode</code> with <code>left</code>,
     * <code>right</code> and <code>distance</code> attributes initialized
     * to <code>left</code>, <code>right</code> and <code>distance</code>
     * argument values, respectively.
     *
     * @param left <code>int</code> the index for the <code>left</code>
     *        element/node
     * @param right <code>int</code> the index for the <code>right</code>
     *        element/node
     * @param distance <code>double</code> the distance between
     *        <code>left</code> and <code>right</code> elements or nodes
     */
    public ClusterTreeNode(int left, int right, double distance) {
        this.left = left;
        this.right = right;
        this.distance = distance;
    }

    /**
     * Copy constructor. Creates a new <code>ClusterTreeNode</code>
     * initializing <code>left</code>, <code>right</code> and
     * <code>distance</code> attributes with values retrieved from
     * <code>node</code>.
     *
     * @param node the <code>ClusterTreeNode</code> to copy the attribute
     *        values from
     */
    public ClusterTreeNode(ClusterTreeNode node) {
        this(node.left, node.right, node.distance);
    }

    /**
     * Returns the index of the left element or node (left son).
     *
     * @return int the index of the right element/node
     */
    public int getLeft() {
        return left;
    }

    /**
     * Returns the index of the right element or node (right son).
     *
     * @return int the index of the right element/node
     */
    public int getRight() {
        return right;
    }

    /**
     * Returns the distance between the left and the right elements/nodes
     * (the sons of this node).
     *
     * @return double the distance between the left and right sons
     */
    public double getDistance() {
        return distance;
    }

    /**
     * Sets the index for left son.
     *
     * @param newLeft <code>int</code> the index for left son
     */
    public void setLeft(int newLeft) {
        left = newLeft;
    }

    /**
     * Sets the index for right son.
     *
     * @param newRight <code>int</code> the index for right son
     */
    public void setRight(int newRight) {
        right = newRight;
    }

    /**
     * Sets the distance between left and right sons.
     *
     * @param newDistance <code>double</code> the distance between left and right sons
     */
    public void setDistance(double newDistance) {
        distance = newDistance;
    }

    /**
     * Overrides <code>Object</code>'s <code>equals</code> method.
     * Checks if this <code>ClusterTreeNode</code> is equal to
     * <code>obj</code>.
     *
     * @param obj the <code>Object</code> to compare with this one
     * @return boolean <code>true</code> if this <codeClusterTreeNode</code>
     *         and <code>obj</code> are equal, <code>false</code> otherwise.
     */
    public boolean equals(Object obj) {
        if (obj != null && obj instanceof ClusterTreeNode) {
            ClusterTreeNode node = (ClusterTreeNode)obj;
            return (this.left == node.left
                    && this.right == node.right
                    && this.distance == node.distance);
        }
        return false;
    }

    /**
     * Overrides <code>Object</code>'s <code>clone</code> method.
     * Clones this <code>ClusterTreeNode</code> object.
     * Returns a <code>ClusterTreeNode</code> object equal to this one.
     *
     * @return Object the clone of this <code>ClusterTreeNode</code>
     */
    public Object clone() {
        return (new ClusterTreeNode(this));
    }
}
